@@ -80,7 +80,6 @@ static void
 write_part_bodystructure (GMimeObject *part, FILE *fp)
 {
 	GMimeParam *param;
-	GList *l;
 	
 	fputc ('(', fp);
 	
@@ -104,10 +103,14 @@ write_part_bodystructure (GMimeObject *part, FILE *fp)
 	}
 	
 	if (GMIME_IS_MULTIPART (part)) {
-		l = GMIME_MULTIPART (part)->subparts;
-		while (l != NULL) {
-			write_part_bodystructure (l->data, fp);
-			l = l->next;
+		GMimeMultipart *multipart = (GMimeMultipart *) part;
+		GMimeObject *subpart;
+		int i, n;
+		
+		n = g_mime_multipart_get_count (multipart);
+		for (i = 0; i < n; i++) {
+			subpart = g_mime_multipart_get_part (multipart, i);
+			write_part_bodystructure (subpart, fp);
 		}
 	} else if (GMIME_IS_MESSAGE_PART (part)) {
 		GMimeMessage *message;
@@ -119,7 +122,7 @@ write_part_bodystructure (GMimeObject *part, FILE *fp)
 		/* print envelope */
 		fputc ('(', fp);
 		
-		nstring = g_mime_message_get_date_string (message);
+		nstring = g_mime_message_get_date_as_string (message);
 		fprintf (fp, "\"%s\" ", nstring);
 		g_free (nstring);
 		
@@ -272,10 +275,12 @@ static void
 write_part (GMimeObject *part, const char *uid, const char *spec)
 {
 	GMimeStream *istream, *ostream;
+	GMimeMultipart *multipart;
+	GMimeMessage *message;
+	GMimeObject *subpart;
 	char *buf, *id;
-	GList *l;
 	FILE *fp;
-	int i;
+	int i, n;
 	
 	buf = g_strdup_printf ("%s/%s.HEADER", uid, spec);
 	fp = fopen (buf, "wt");
@@ -288,20 +293,18 @@ write_part (GMimeObject *part, const char *uid, const char *spec)
 	fclose (fp);
 	
 	if (GMIME_IS_MULTIPART (part)) {
+		multipart = (GMimeMultipart *) part;
 		buf = g_alloca (strlen (spec) + 14);
 		id = g_stpcpy (buf, spec);
 		*id++ = '.';
-		i = 1;
-		l = GMIME_MULTIPART (part)->subparts;
-		while (l != NULL) {
-			sprintf (id, "%d", i);
-			write_part (l->data, uid, buf);
-			l = l->next;
-			i++;
+		
+		n = g_mime_multipart_get_count (multipart);
+		for (i = 0; i < n; i++) {
+			subpart = g_mime_multipart_get_part (multipart, i);
+			sprintf (id, "%d", i + 1);
+			write_part (subpart, uid, buf);
 		}
 	} else if (GMIME_IS_MESSAGE_PART (part)) {
-		GMimeMessage *message;
-		
 		buf = g_strdup_printf ("%s/%s.TEXT", uid, spec);
 		fp = fopen (buf, "wt");
 		g_free (buf);
@@ -750,16 +753,15 @@ reconstruct_part_content (GMimePart *part, const char *uid, const char *spec)
 static void
 reconstruct_message_part (GMimeMessagePart *msgpart, const char *uid, const char *spec)
 {
+	GMimeMessage *message;
 	GMimeParser *parser;
 	GMimeStream *stream;
 	char *filename;
 	int fd;
 	
-	if (msgpart->message)
-		g_object_unref (msgpart->message);
-	
 	filename = g_strdup_printf ("%s/%s.TEXT", uid, spec);
 	if ((fd = open (filename, O_RDONLY))) {
+		g_mime_message_part_set_message (msgpart, NULL);
 		g_free (filename);
 		return;
 	}
@@ -771,8 +773,11 @@ reconstruct_message_part (GMimeMessagePart *msgpart, const char *uid, const char
 	g_mime_parser_set_scan_from (parser, FALSE);
 	g_object_unref (stream);
 	
-	msgpart->message = g_mime_parser_construct_message (parser);
+	message = g_mime_parser_construct_message (parser);
 	g_object_unref (parser);
+	
+	g_mime_message_part_set_message (msgpart, message);
+	g_object_unref (message);
 }
 
 static void
@@ -834,7 +839,7 @@ reconstruct_multipart (GMimeMultipart *multipart, struct _bodystruct *body,
 			reconstruct_part_content ((GMimePart *) subpart, uid, subspec);
 		}
 		
-		g_mime_multipart_add_part (multipart, subpart);
+		g_mime_multipart_add (multipart, subpart);
 		g_object_unref (subpart);
 		
 		part = part->next;
