@@ -174,14 +174,6 @@ static GStaticMutex charset_lock = G_STATIC_MUTEX_INIT;
 #endif /* G_THREADS_ENABLED */
 
 
-static void
-iconv_charset_free (gpointer key, gpointer value, gpointer user_data)
-{
-	g_free (key);
-	g_free (value);
-}
-
-
 /**
  * g_mime_charset_map_shutdown:
  *
@@ -193,7 +185,6 @@ g_mime_charset_map_shutdown (void)
 	if (!iconv_charsets)
 		return;
 	
-	g_hash_table_foreach (iconv_charsets, iconv_charset_free, NULL);
 	g_hash_table_destroy (iconv_charsets);
 	iconv_charsets = NULL;
 	
@@ -264,7 +255,7 @@ g_mime_charset_map_init (void)
 	if (iconv_charsets)
 		return;
 	
-	iconv_charsets = g_hash_table_new (g_str_hash, g_str_equal);
+	iconv_charsets = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	
 	for (i = 0; known_iconv_charsets[i].charset != NULL; i++) {
 		charset = g_ascii_strdown (known_iconv_charsets[i].charset, -1);
@@ -324,7 +315,7 @@ g_mime_charset_map_init (void)
  *
  * Gets the user's locale charset (or iso-8859-1 by default).
  *
- * Returns the user's locale charset (or iso-8859-1 by default).
+ * Returns: the user's locale charset (or iso-8859-1 by default).
  **/
 const char *
 g_mime_locale_charset (void)
@@ -343,7 +334,7 @@ g_mime_locale_charset (void)
  *
  * Gets the user's locale language code (or %NULL by default).
  *
- * Returns the user's locale language code (or %NULL by default).
+ * Returns: the user's locale language code (or %NULL by default).
  **/
 const char *
 g_mime_locale_language (void)
@@ -365,7 +356,7 @@ g_mime_locale_language (void)
  * @charset. Currently only handles CJK and Russian/Ukranian
  * charset->lang mapping. Everything else will return %NULL.
  *
- * Returns a language code that is specific to @charset, or %NULL on
+ * Returns: a language code that is specific to @charset, or %NULL on
  * fail.
  **/
 const char *
@@ -405,7 +396,7 @@ strdown (char *str)
  *
  * Attempts to find an iconv-friendly charset name for @charset.
  *
- * Returns an iconv-friendly charset name for @charset.
+ * Returns: an iconv-friendly charset name for @charset.
  **/
 const char *
 g_mime_charset_iconv_name (const char *charset)
@@ -538,7 +529,7 @@ static const char *windows_charsets[] = {
  * the canonical ISO charset names (such as using ISO8859-1 rather
  * than the canonical form ISO-8859-1).
  *
- * Returns a canonical charset name for @charset.
+ * Returns: a canonical charset name for @charset.
  **/
 const char *
 g_mime_charset_canon_name (const char *charset)
@@ -590,7 +581,7 @@ g_mime_charset_canon_name (const char *charset)
  * Note: This function is deprecated. Use g_mime_charset_iconv_name()
  * instead.
  *
- * Returns an iconv-friendly charset name for @charset.
+ * Returns: an iconv-friendly charset name for @charset.
  **/
 const char *
 g_mime_charset_name (const char *charset)
@@ -607,7 +598,7 @@ g_mime_charset_name (const char *charset)
  * Note: This function is deprecated. Use g_mime_locale_charset()
  * instead.
  *
- * Returns the user's locale charset (or iso-8859-1 by default).
+ * Returns: the user's locale charset (or iso-8859-1 by default).
  **/
 const char *
 g_mime_charset_locale_name (void)
@@ -623,7 +614,7 @@ g_mime_charset_locale_name (void)
  * Maps the ISO-8859-# charset to the equivalent Windows-CP125#
  * charset.
  *
- * Returns equivalent Windows charset.
+ * Returns: equivalent Windows charset.
  **/
 const char *
 g_mime_charset_iso_to_windows (const char *isocharset)
@@ -694,8 +685,8 @@ g_mime_charset_init (GMimeCharset *charset)
 /**
  * g_mime_charset_step:
  * @charset: charset structure
- * @in: input text buffer (must be in UTF-8)
- * @len: input buffer length
+ * @inbuf: input text buffer (must be in UTF-8)
+ * @inlen: input buffer length
  *
  * Steps through the input buffer 1 unicode character (glyph) at a
  * time (ie, not necessarily 1 byte at a time). Bitwise 'and' our
@@ -703,10 +694,10 @@ g_mime_charset_init (GMimeCharset *charset)
  * limiting what charsets our @charset->mask can match.
  **/
 void
-g_mime_charset_step (GMimeCharset *charset, const char *in, size_t len)
+g_mime_charset_step (GMimeCharset *charset, const char *inbuf, size_t inlen)
 {
-	register const char *inptr = in;
-	const char *inend = in + len;
+	register const char *inptr = inbuf;
+	const char *inend = inbuf + inlen;
 	register unsigned int mask;
 	register int level;
 	
@@ -767,7 +758,7 @@ charset_best_mask (unsigned int mask)
  *
  * Gets the best charset name based on the charset mask @charset.
  *
- * Returns a pointer to a string containing the best charset name that
+ * Returns: a pointer to a string containing the best charset name that
  * can represent the charset mask @charset.
  **/
 const char *
@@ -784,21 +775,21 @@ g_mime_charset_best_name (GMimeCharset *charset)
 
 /**
  * g_mime_charset_best:
- * @in: a UTF-8 text buffer
- * @inlen: length of @in
+ * @inbuf: a UTF-8 text buffer
+ * @inlen: input buffer length
  *
  * Computes the best charset to use to encode this text buffer.
  *
- * Returns the charset name best suited for the input text or %NULL if
+ * Returns: the charset name best suited for the input text or %NULL if
  * it is US-ASCII safe.
  **/
 const char *
-g_mime_charset_best (const char *in, size_t inlen)
+g_mime_charset_best (const char *inbuf, size_t inlen)
 {
 	GMimeCharset charset;
 	
 	g_mime_charset_init (&charset);
-	g_mime_charset_step (&charset, in, inlen);
+	g_mime_charset_step (&charset, inbuf, inlen);
 	
 	return g_mime_charset_best_name (&charset);
 }
@@ -813,7 +804,7 @@ g_mime_charset_best (const char *in, size_t inlen)
  *
  * Check to see if the UTF-8 @text will fit safely within @charset.
  *
- * Returns %TRUE if it is safe to encode @text into @charset or %FALSE
+ * Returns: %TRUE if it is safe to encode @text into @charset or %FALSE
  * otherwise.
  **/
 gboolean
@@ -923,7 +914,7 @@ g_mime_set_user_charsets (const char **charsets)
  * Get the list of user-preferred charsets set with
  * g_mime_set_user_charsets().
  *
- * Returns an array of user-set charsets or %NULL if none set.
+ * Returns: an array of user-set charsets or %NULL if none set.
  **/
 const char **
 g_mime_user_charsets (void)
