@@ -482,7 +482,7 @@ g_mime_filter_html_new (guint32 flags, guint32 colour)
 	GMimeFilterHTML *new;
 	guint i;
 	
-	new = g_object_new (GMIME_TYPE_FILTER_HTML, NULL);
+	new = g_object_newv (GMIME_TYPE_FILTER_HTML, 0, NULL);
 	new->flags = flags;
 	new->colour = colour;
 	
