@@ -234,7 +234,7 @@ g_mime_message_part_new (const char *subtype)
 	GMimeContentType *content_type;
 	GMimeMessagePart *part;
 	
-	part = g_object_new (GMIME_TYPE_MESSAGE_PART, NULL);
+	part = g_object_newv (GMIME_TYPE_MESSAGE_PART, 0, NULL);
 	
 	content_type = g_mime_content_type_new ("message", subtype ? subtype : "rfc822");
 	g_mime_object_set_content_type (GMIME_OBJECT (part), content_type);
