@@ -80,14 +80,15 @@ struct _GMimeObjectClass {
 
 
 /**
- * GMimePartFunc:
- * @part: A MIME part.
+ * GMimeObjectForeachFunc:
+ * @parent: parent #GMimeObject
+ * @part: a #GMimeObject
  * @user_data: User-supplied callback data.
  *
- * The function signature for a callback to
- * g_mime_message_foreach_part() and g_mime_multipart_foreach().
+ * The function signature for a callback to g_mime_message_foreach()
+ * and g_mime_multipart_foreach().
  **/
-typedef void (* GMimePartFunc) (GMimeObject *part, gpointer user_data);
+typedef void (* GMimeObjectForeachFunc) (GMimeObject *parent, GMimeObject *part, gpointer user_data);
 
 
 GType g_mime_object_get_type (void);
@@ -97,7 +98,7 @@ void g_mime_object_register_type (const char *type, const char *subtype, GType o
 GMimeObject *g_mime_object_new (GMimeContentType *content_type);
 GMimeObject *g_mime_object_new_type (const char *type, const char *subtype);
 
-void g_mime_object_set_content_type (GMimeObject *object, GMimeContentType *mime_type);
+void g_mime_object_set_content_type (GMimeObject *object, GMimeContentType *content_type);
 GMimeContentType *g_mime_object_get_content_type (GMimeObject *object);
 void g_mime_object_set_content_type_parameter (GMimeObject *object, const char *name, const char *value);
 const char *g_mime_object_get_content_type_parameter (GMimeObject *object, const char *name);
